/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is regenerated.
 */

import * as moment from "moment";

export interface MultiLanguageInput {
  /**
   * This is the 2 letter ISO 639-1 representation of a language. For example, use "en" for
   * English; "es" for Spanish etc.,
  */
  language?: string;
  /**
   * Unique, non-empty document identifier.
  */
  id?: string;
  text?: string;
}

export interface MultiLanguageBatchInput {
  documents?: MultiLanguageInput[];
}

export interface MatchRecord {
  /**
   * (optional) If a well-known item with Wikipedia link is recognized, a decimal number denoting
   * the confidence level of the Wikipedia info will be returned.
  */
  wikipediaScore?: number;
  /**
   * (optional) If an entity type is recognized, a decimal number denoting the confidence level of
   * the entity type will be returned.
  */
  entityTypeScore?: number;
  /**
   * Entity text as appears in the request.
  */
  text?: string;
  /**
   * Start position (in Unicode characters) for the entity match text.
  */
  offset?: number;
  /**
   * Length (in Unicode characters) for the entity match text.
  */
  length?: number;
}

export interface EntityRecord {
  /**
   * Entity formal name.
  */
  name?: string;
  /**
   * List of instances this entity appears in the text.
  */
  matches?: MatchRecord[];
  /**
   * Wikipedia language for which the WikipediaId and WikipediaUrl refers to.
  */
  wikipediaLanguage?: string;
  /**
   * Wikipedia unique identifier of the recognized entity.
  */
  wikipediaId?: string;
  /**
   * URL for the entity's Wikipedia page.
  */
  readonly wikipediaUrl?: string;
  /**
   * Bing unique identifier of the recognized entity. Use in conjunction with the Bing Entity
   * Search API to fetch additional relevant information.
  */
  bingId?: string;
  /**
   * Entity type from Named Entity Recognition model
  */
  type?: string;
  /**
   * Entity sub type from Named Entity Recognition model
  */
  subType?: string;
}

export interface DocumentStatistics {
  /**
   * Number of text elements recognized in the document.
  */
  charactersCount?: number;
  /**
   * Number of transactions for the document.
  */
  transactionsCount?: number;
}

export interface EntitiesBatchResultItem {
  /**
   * Unique, non-empty document identifier.
  */
  id?: string;
  /**
   * Recognized entities in the document.
  */
  readonly entities?: EntityRecord[];
  /**
   * (Optional) if showStats=true was specified in the request this field will contain information
   * about the document payload.
  */
  statistics?: DocumentStatistics;
}

export interface ErrorRecord {
  /**
   * Input document unique identifier the error refers to.
  */
  id?: string;
  /**
   * Error message.
  */
  message?: string;
}

export interface RequestStatistics {
  /**
   * Number of documents submitted in the request.
  */
  documentsCount?: number;
  /**
   * Number of valid documents. This excludes empty, over-size limit or non-supported languages
   * documents.
  */
  validDocumentsCount?: number;
  /**
   * Number of invalid documents. This includes empty, over-size limit or non-supported languages
   * documents.
  */
  erroneousDocumentsCount?: number;
  /**
   * Number of transactions for the request.
  */
  transactionsCount?: number;
}

export interface EntitiesBatchResult {
  /**
   * Response by document
  */
  readonly documents?: EntitiesBatchResultItem[];
  /**
   * Errors and Warnings by document
  */
  readonly errors?: ErrorRecord[];
  /**
   * (Optional) if showStats=true was specified in the request this field will contain information
   * about the request payload.
  */
  readonly statistics?: RequestStatistics;
}

export interface InternalError {
  code?: string;
  message?: string;
  innerError?: InternalError;
}

export interface ErrorResponse {
  code?: string;
  message?: string;
  target?: string;
  innerError?: InternalError;
}

export interface KeyPhraseBatchResultItem {
  /**
   * Unique, non-empty document identifier.
  */
  id?: string;
  /**
   * A list of representative words or phrases. The number of key phrases returned is proportional
   * to the number of words in the input document.
  */
  readonly keyPhrases?: string[];
  /**
   * (Optional) if showStats=true was specified in the request this field will contain information
   * about the document payload.
  */
  statistics?: DocumentStatistics;
}

export interface KeyPhraseBatchResult {
  /**
   * Response by document
  */
  readonly documents?: KeyPhraseBatchResultItem[];
  /**
   * Errors and Warnings by document
  */
  readonly errors?: ErrorRecord[];
  /**
   * =(Optional) if showStats=true was specified in the request this field will contain information
   * about the request payload.
  */
  readonly statistics?: RequestStatistics;
}

export interface LanguageInput {
  countryHint?: string;
  /**
   * Unique, non-empty document identifier.
  */
  id?: string;
  text?: string;
}

export interface LanguageBatchInput {
  documents?: LanguageInput[];
}

export interface DetectedLanguage {
  /**
   * Long name of a detected language (e.g. English, French).
  */
  name?: string;
  /**
   * A two letter representation of the detected language according to the ISO 639-1 standard (e.g.
   * en, fr).
  */
  iso6391Name?: string;
  /**
   * A confidence score between 0 and 1. Scores close to 1 indicate 100% certainty that the
   * identified language is true.
  */
  score?: number;
}

export interface LanguageBatchResultItem {
  /**
   * Unique, non-empty document identifier.
  */
  id?: string;
  /**
   * A list of extracted languages.
  */
  detectedLanguages?: DetectedLanguage[];
  /**
   * (Optional) if showStats=true was specified in the request this field will contain information
   * about the document payload.
  */
  statistics?: DocumentStatistics;
}

export interface LanguageBatchResult {
  /**
   * Response by document
  */
  readonly documents?: LanguageBatchResultItem[];
  /**
   * Errors and Warnings by document
  */
  readonly errors?: ErrorRecord[];
  /**
   * (Optional) if showStats=true was specified in the request this field will contain information
   * about the request payload.
  */
  readonly statistics?: RequestStatistics;
}

export interface SentimentBatchResultItem {
  /**
   * Unique, non-empty document identifier.
  */
  id?: string;
  /**
   * A decimal number between 0 and 1 denoting the sentiment of the document. A score above 0.7
   * usually refers to a positive document while a score below 0.3 normally has a negative
   * connotation. Mid values refer to neutral text.
  */
  score?: number;
  /**
   * (Optional) if showStats=true was specified in the request this field will contain information
   * about the document payload.
  */
  statistics?: DocumentStatistics;
}

export interface SentimentBatchResult {
  /**
   * Response by document
  */
  readonly documents?: SentimentBatchResultItem[];
  /**
   * Errors and Warnings by document
  */
  readonly errors?: ErrorRecord[];
  /**
   * (Optional) if showStats=true was specified in the request this field will contain information
   * about the request payload.
  */
  readonly statistics?: RequestStatistics;
}
